# -*- coding: UTF-8 -*-
from datetime import datetime
from StringIO import StringIO
import pandas as pd
from utils import get_station


PARSE_EVENT_CODE = {
    0: [1, 1],
    1: [1, 2],
    2: [1, 3],
    3: [1, 98],
    4: [1, 98],
    5: [4, 1],
    6: [1, 30],
    7: [1, 99],
    8: [1, 99],
    9: [1, 50],
    10: [2, 2],
    11: [2, 3],
    12: [1, 9],
    13: [6, 1],
    14: [1, 98],
    15: [1, 8],
    16: [5, 20],
}


class PlcMeasureEventsParser(object):

    def __init__(self, events_file, cnc_name):
        """Creates a dictionary with data from PLC800"""
        # Support for files or raw string
        if isinstance(events_file, str):
            events_file = StringIO(events_file)
        if not hasattr(events_file, 'read'):
            raise TypeError('Must be file pointer or basestring')
        # read file
        self.cnc_name = cnc_name
        self.df_profiles = pd.read_csv(events_file, header=None, sep=';',
                                       names=['name', 'timestamp', 'event_group', 'event_code_desc'],
                                       dtype={'name': str, 'timestamp': str, 'event_group': int,
                                              'event_code_desc': str})

    @property
    def events(self):
        df = self.df_profiles.copy()
        df['cnc_name'] = self.cnc_name
        df['timestamp'] = df['timestamp'].apply(
            lambda x: datetime.strptime(x, '%d/%m/%Y %H:%M:%S').strftime('%Y-%m-%d %H:%M:%S'))
        df['season'] = df['timestamp'].apply(lambda x: get_station(x))
        df['event_code'] = df['event_group'].apply(lambda x: PARSE_EVENT_CODE.get(x)[1])
        df['event_group'] = df['event_group'].apply(lambda x: PARSE_EVENT_CODE.get(x)[0])
        df['type'] = 'M'
        df['active'] = True
        return df.T.to_dict().values()



# -*- coding: UTF-8 -*-
from StringIO import StringIO
import pandas as pd
from datetime import datetime, timedelta


class PlcMonthlyBillingsParser(object):
    """PLC Monthly Billings Parser"""

    def __init__(self, billings_file, cnc_name):
        """Creates a dictionary with data from PLC800"""
        # Support for files or raw string
        if isinstance(billings_file, str):
            billings_file = StringIO(billings_file)
        if not hasattr(billings_file, 'read'):
            raise TypeError('Must be file pointer or basestring')
        # read file
        self.cnc_name = cnc_name
        df = pd.read_csv(billings_file, header=None, sep='\n')
        self.df_billings = df[0].str.split(';', expand=True)

    @property
    def billings(self):
        data = []
        for row in self.df_billings.T.to_dict().values():
            num_periods = int(row[2]) + 1
            for period in range(0, num_periods):
                # read row info for each period
                meter_name = row[0]
                contract = int(row[1])
                # date_end is last day of month
                date_end_dt = datetime.strptime(row[3], '%d/%m/%Y %H:%M:%S')
                date_end = date_end_dt.strftime('%Y-%m-%d %H:%M:%S')
                # get first day of month
                date_begin_dt = datetime.strptime(date_end, '%Y-%m-%d %H:%M:%S')
                date_begin = date_begin_dt.replace(day=1).strftime('%Y-%m-%d %H:%M:%S')
                # add 1 day to date_end to get first day of next month
                date_end_dt = date_end_dt + timedelta(days=1)
                date_end = date_end_dt.strftime('%Y-%m-%d %H:%M:%S')
                # energy
                index_activa = 4 + period
                activa_entrante = int(row[index_activa])
                index_inductiva = index_activa + num_periods
                inductiva = int(row[index_inductiva])
                index_capacitativa = index_inductiva + num_periods
                capacitativa = int(row[index_capacitativa])
                index_maxim = index_capacitativa + num_periods
                maxim = int(float(row[index_maxim])*1000)
                billing = {
                    'name': meter_name,
                    'contract': contract,
                    'type': 'month',
                    'value': 'a',
                    'date_begin': date_begin,
                    'date_end': date_end,
                    'period': period,
                    'max': maxim,
                    'date_max': False,
                    'ai': activa_entrante,
                    'ae': 0,
                    'r1': inductiva,
                    'r2': 0,
                    'r3': 0,
                    'r4': capacitativa,
                    'cnc_name': self.cnc_name,
                }
                data.append(billing)
        return data


class PlcDailyBillingsParser(object):
    """PLC Daily Billings Parser"""

    def __init__(self, billings_file, cnc_name):
        """Creates a dictionary with data from PLC800"""
        # Support for files or raw string
        if isinstance(billings_file, str):
            billings_file = StringIO(billings_file)
        if not hasattr(billings_file, 'read'):
            raise TypeError('Must be file pointer or basestring')
        # read file
        self.cnc_name = cnc_name
        self.df_billings = pd.read_csv(billings_file, header=None, sep=';',
                                       names=['name', 'date_end', 'ai', 'ae', 'r1', 'r2', 'r3', 'r4'],
                                       dtype={'name': str, 'date_end': str, 'ai': int, 'ae': int,
                                              'r1': int, 'r2': int, 'r3': int, 'r4': int})

    @property
    def billings(self):
        df = self.df_billings.copy()
        df['contract'] = 1
        df['type'] = 'day'
        df['value'] = 'a'
        df['contract'] = 1
        df['date_end'] = df['date_end'].apply(
            lambda x: datetime.strptime(x, '%d/%m/%Y %H:%M:%S').strftime('%Y-%m-%d %H:%M:%S'))
        df['date_begin'] = df['date_end']
        df['period'] = 0
        df['max'] = 0
        df['date_max'] = False
        df['cnc_name'] = self.cnc_name
        return df.T.to_dict().values()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from datetime import datetime
from pytz import timezone


def get_station(dt_str):
    dt = datetime.strptime(dt_str, '%Y-%m-%d %H:%M:%S')
    if not dt.tzinfo:
        dt = timezone('Europe/Madrid').localize(dt)
    if timezone('Europe/Madrid').normalize(dt).dst():
        return 'S'
    else:
        return 'W'


def assert_list_dict_eq(list_dict_a, list_dict_b, sorting_key):
    """ Asserts that two lists of dicts are equal sorting by key """
    if len(list_dict_a) != len(list_dict_b):
        raise AssertionError()
    list_dict_a_sorted = sorted(list_dict_a, key=lambda k: k[sorting_key])
    list_dict_b_sorted = sorted(list_dict_b, key=lambda k: k[sorting_key])
    for index_dict in range(0, len(list_dict_a_sorted)):
        if list_dict_a_sorted[index_dict] != list_dict_b_sorted[index_dict]:
            raise AssertionError()




# -*- coding: UTF-8 -*-
from datetime import datetime
from StringIO import StringIO
import pandas as pd
from utils import get_station


class PlcProfilesParser(object):

    def __init__(self, profiles_file, cnc_name):
        """Creates a dictionary with data from PLC800"""
        # Support for files or raw string
        if isinstance(profiles_file, str):
            profiles_file = StringIO(profiles_file)
        if not hasattr(profiles_file, 'read'):
            raise TypeError('Must be file pointer or basestring')
        # read file
        self.cnc_name = cnc_name
        self.df_profiles = pd.read_csv(profiles_file, header=None, sep=';',
                                       names=['name', 'timestamp', 'flag', 'ai', 'ae', 'r1', 'r2', 'r3', 'r4'],
                                       dtype={'name': str, 'timestamp': str, 'ai': int, 'ae': int,
                                              'r1': int, 'r2': int, 'r3': int, 'r4': int})

    @property
    def profiles(self):
        df = self.df_profiles.drop(columns=['flag'])
        df['cnc_name'] = self.cnc_name
        df['magn'] = 1000
        df['timestamp'] = df['timestamp'].apply(
            lambda x: datetime.strptime(x, '%d/%m/%Y %H:%M:%S').strftime('%Y-%m-%d %H:%M:%S'))
        df['season'] = df['timestamp'].apply(lambda x: get_station(x))
        return df.T.to_dict().values()



from setuptools import setup, find_packages


setup(
    name='plc800stg',
    version='0.0.2',
    packages=find_packages(),
    url='https://github.com/gisce/plc800stg',
    license='MIT License',
    author='GISCE-TI, S.L.',
    author_email='devel@gisce.net',
    install_requires=[
        'lxml',
        'pandas',
    ],
    description='PLC800 STG'
)
